package com.letmetrial.api.auth.usecase;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.springframework.stereotype.Component;

import com.letmetrial.api.auth.client.keycloak.KeycloakClient;
import com.letmetrial.api.auth.client.keycloak.dto.CredentialDTO;
import com.letmetrial.api.auth.client.keycloak.dto.UserDTO;
import com.letmetrial.api.auth.client.keycloak.dto.UserRegisterDTO;

import jakarta.validation.constraints.Email;
import jakarta.validation.constraints.NotBlank;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
public class CadastrarContaMedicoUC {

    private final CadastrarContaUCMapper mapper;

    private final KeycloakClient keycloakClient;

    public UsuarioDTO executar(NovaContaDTO novaConta) {
        return mapper.toUsuarioDTO(keycloakClient.criarUsuario(mapper.toUserRegisterDTO(novaConta, true, "medicos")));
    }

    public record NovaContaDTO(
            @Email String email,
            @NotBlank String credencial) {
    }

    public record UsuarioDTO(
            UUID id,
            String email) {
    }

    @Mapper
    public interface CadastrarContaUCMapper {

        UsuarioDTO toUsuarioDTO(UserDTO userDTO);

        @Mapping(target = "credentials", source = "novaContaDTO.credencial")
        @Mapping(target = "groups", source = "grupos")
        UserRegisterDTO toUserRegisterDTO(NovaContaDTO novaContaDTO, boolean enabled, String... grupos);

        default List<CredentialDTO> toCredentialDTOList(String credential) {
            return new ArrayList<>(List.of(new CredentialDTO("password", credential, false)));
        }

        default List<String> toGroups(String[] grupos) {
            return grupos == null ? null : new ArrayList<>(List.of(grupos));
        }

    }

}
